/*
 * Mirai. Copyright (C) 2024 Vyn
 * This file is licensed under version 3 of the GNU General Public License (GPL-3.0-only)
 * The license can be found in the LICENSE file or at https://www.gnu.org/licenses/gpl-3.0.txt
 */

#ifndef CPP_UTILS_VECTOR_H
#define CPP_UTILS_VECTOR_H

#include <algorithm>
#include <vector>

namespace cpputils::vector
{

template <typename C, typename T>
concept AnyIterable = std::same_as<typename C::value_type, T> && requires(C c) {
	{
		c.begin()
	} -> std::forward_iterator;
	{
		c.end()
	} -> std::forward_iterator;
	{
		const_cast<const C &>(c).begin()
	} -> std::forward_iterator;
	{
		const_cast<const C &>(c).end()
	} -> std::forward_iterator;
};

template <typename T> bool contains(const std::vector<T> &vec, const T &value)
{
	return std::ranges::find(vec, value) != vec.end();
}

template <typename T> bool containsAll(const std::vector<T> &vec, const AnyIterable<T> auto vec2)
{
	for (auto &elem : vec) {
		if (!contains(vec2, elem)) {
			return false;
		}
	}
	return true;
}
} // namespace cpputils::vector

#endif
