/*
 * Mirai. Copyright (C) 2024 Vyn
 * This file is licensed under version 3 of the GNU General Public License (GPL-3.0-only)
 * The license can be found in the LICENSE file or at https://www.gnu.org/licenses/gpl-3.0.txt
 */

#pragma once

#include "event_emitter.h"
#include "source.h"
#include <functional>
#include <memory>
#include <optional>
#include <string>

namespace mirai
{

class core
{

  public:
	core(const std::string &configFilePath);
	void add_source(
		const std::string &name, const std::string &color, const std::string &type,
		std::unique_ptr<data_provider> &&source
	);
	void
	edit_source(int id, const std::string &name, const std::string &color, const std::string &path);
	void remove_source(int id);
	void unload_all_sources();
	void save();

	std::vector<std::unique_ptr<source>> &get_sources();

	// Returns a non owning pointer to the requested resource or nullptr if not found.
	source *get_source_by_id(int id);
	source *get_source_by_name(const std::string &name);

	void on_source_added(std::function<void(source *)> f);
	void on_source_edited(std::function<void(source *)> f);
	void on_source_removed(std::function<void(int)> f);

  private:
	void load_config(const std::string &path);
	void save_config();
	std::vector<std::unique_ptr<source>> _sources;
	std::string _config_path;

	event_emitter<source *> source_added;
	event_emitter<source *> source_edited;
	event_emitter<int> source_removed;
};
} // namespace mirai
