/*
 * Mirai. Copyright (C) 2024 Vyn
 * This file is licensed under version 3 of the GNU General Public License (GPL-3.0-only)
 * The license can be found in the LICENSE file or at https://www.gnu.org/licenses/gpl-3.0.txt
 */

#pragma once

#include <bits/chrono.h>
#include <chrono>
#include <ctime>
#include <optional>
#include <string>

namespace mirai
{

struct date {
	explicit date(int year, unsigned month, unsigned day);

	explicit date(std::chrono::time_point<std::chrono::system_clock> tp);
	explicit date(std::chrono::year_month_day chronoDate);

	bool operator==(const date &other) const;
	bool operator<(const date &other) const;
	bool operator>(const date &other) const;
	bool operator>=(const date &other) const;
	bool operator<=(const date &other) const;

	std::chrono::year_month_day to_std_chrono() const
	{
		return std::chrono::year_month_day{
			std::chrono::year(year),
			std::chrono::month(month),
			std::chrono::day(day),
		};
	}

	int year;
	unsigned month;
	unsigned day;
};

struct time {

	explicit time(int hour, int minute);
	explicit time(std::chrono::time_point<std::chrono::system_clock> tp);

	bool operator==(const time &other) const;
	bool operator<(const time &other) const;
	bool operator>(const time &other) const;

	int hour;
	int minute;
};

std::optional<class date> string_to_date(const std::string &dateStr);

} // namespace mirai
