/*
 * Mirai. Copyright (C) 2024 Vyn
 * This file is licensed under version 3 of the GNU General Public License (GPL-3.0-only)
 * The license can be found in the LICENSE file or at https://www.gnu.org/licenses/gpl-3.0.txt
 */

#include "event.h"
#include "task.h"
#include <string>

namespace mirai
{

int event::id() const
{
	return _data.id;
}

int event::source_id() const
{
	return _provider->id;
}

std::string event::title() const
{
	return _data.title;
}

date event::date() const
{
	return _data.date;
}

time event::starts_at() const
{
	return _data.starts_at;
}

time event::ends_at() const
{
	return _data.ends_at;
}

void event::set_title(const std::string &newTitle)
{
	_provider->update_event(id(), {.title = newTitle});
}

void event::set_date(const class date &date)
{
	_provider->update_event(id(), {.date = date});
}

void event::set_start_time(const time &time)
{
	_provider->update_event(id(), {.starts_at = time});
}

void event::set_end_time(const time &time)
{
	_provider->update_event(id(), {.ends_at = time});
}
} // namespace mirai
