/*
 * Mirai. Copyright (C) 2024 Vyn
 * This file is licensed under version 3 of the GNU General Public License (GPL-3.0-only)
 * The license can be found in the LICENSE file or at https://www.gnu.org/licenses/gpl-3.0.txt
 */

#pragma once

#include "mirai-core/core.h"
#include "mirai-core/source.h"
#include "slint.h"
#include "ui.h"
#include <unordered_map>
#include <vector>

class app_logic
{

  public:
	app_logic(mirai::core *mirai);

	void run();

	void update_views();
	void update_sidebar_view();
	void update_calendar_view();
	void update_tasks_view();
	void update_available_sources();

	void show_all_sources();
	void show_source(const mirai::source &source);
	void hide_source(const mirai::source &source);
	void hide_all_sources();
	bool should_show_source(const mirai::source &source) const;

	void show_completed_tasks();
	void hide_completed_tasks();
	bool should_hide_completed_tasks() const;

  private:
	void setup_callbacks();
	const ui::AppModels &models();
	const ui::AppActions &actions();

	ui::SidebarView _sidebar_view;
	ui::TasksView _tasks_view;
	ui::CalendarView _calendar_view;

	slint::ComponentHandle<ui::AppWindow> _main_window = ui::AppWindow::create();

	mirai::core *_mirai_core;

	std::unordered_map<int, bool> _source_filters;
	bool _should_hide_completed_tasks;
};
